@@ -30,6 +30,7 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtkvbbox.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb-extra/libgnomedb-extra.h>
 #include <libgnomedb/gnome-db-util.h>
 #include "provider-config.h"
 
@@ -59,9 +60,13 @@ show_providers (ProviderConfigPrivate *priv)
 	for (l = prv_list; l != NULL; l = l->next) {
 		GList *value_list;
 		GdaProviderInfo *prv_info = (GdaProviderInfo *) l->data;
+		GValue *tmpval;
 
-		value_list = g_list_append (NULL, gda_value_new_string (prv_info->id));
-		value_list = g_list_append (value_list, gda_value_new_string (prv_info->description));
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), prv_info->id);
+		value_list = g_list_append (NULL, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), prv_info->description);
+		value_list = g_list_append (value_list, tmpval);
 
 		gda_data_model_append_values (model, (const GList *) value_list, NULL);
 
@@ -88,6 +93,7 @@ provider_config_new (void)
 	GtkWidget *box;
 	GtkWidget *button;
 	GtkWidget *label;
+	GtkWidget *sw;
 	gchar *title;
 
 	priv = g_new0 (ProviderConfigPrivate, 1);
@@ -113,15 +119,20 @@ provider_config_new (void)
 	gtk_widget_show (priv->title);
 
 	/* create the provider list */
-	priv->provider_list = gnome_db_raw_grid_new (NULL);
-	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->provider_list), 0, FALSE);
-	g_object_set (G_OBJECT (priv->provider_list), "info_cell_visible", FALSE, NULL);
-
-	gtk_widget_show (priv->provider_list);
-	gtk_table_attach (GTK_TABLE (table), priv->provider_list, 0, 1, 1, 2,
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);	
+	gtk_table_attach (GTK_TABLE (table), sw, 0, 1, 1, 2,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
 			  0, 0);
+
+	priv->provider_list = gnome_db_raw_grid_new (NULL);
+	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->provider_list), 0, FALSE);
+	g_object_set (G_OBJECT (priv->provider_list), "info_cell_visible", FALSE, NULL);
+	gtk_container_add (GTK_CONTAINER (sw), priv->provider_list);
+	
+	gtk_widget_show_all (sw);
 	show_providers (priv);
 
 	/* add tip */
