@@ -8,6 +8,7 @@
 #include <glib/gi18n-lib.h>
 
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 #include <demos.h>
 
@@ -37,30 +38,6 @@ struct _CallbackData
 	GtkTreePath *path;
 };
 
-#ifdef G_OS_WIN32
-
-#undef DEMOCODEDIR
-
-static char *
-get_democodedir (void)
-{
-	static char *result = NULL;
-
-	if (result == NULL) {
-		result = g_win32_get_package_installation_directory (NULL, NULL);
-		if (result == NULL)
-			result = "unknown-location";
-		
-		result = g_strconcat (result, "\\share\\gnome-db\\demo", NULL);
-	}
-
-	return result;
-}
-
-#define DEMOCODEDIR get_democodedir ()
-
-#endif
-
 /**
  * demo_find_file:
  * @base: base filename
@@ -81,7 +58,10 @@ demo_find_file (const char *base, GError **err)
 	    g_file_test (base, G_FILE_TEST_EXISTS))
 		return g_strdup (base);
 	else {
-		char *filename = g_build_filename (DEMOCODEDIR, base, NULL);
+		gchar *filename;
+
+		filename = gnome_db_gbr_get_file_path (GNOME_DB_DATA_DIR, LIBGNOMEDB_ABI_NAME, "demo",
+						       base, NULL);
 		if (!g_file_test (filename, G_FILE_TEST_EXISTS)) {
 			g_set_error (err, G_FILE_ERROR, G_FILE_ERROR_NOENT,
 				     "Cannot find demo data file \"%s\"", base);
@@ -760,9 +740,13 @@ main (int argc, char **argv)
 	gchar *full_filename, *dirname;
 	gchar *cncstring;
 	GtkWidget *msg;
+	gchar *str;
 
 	/* Initialize i18n support */
-        bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
+	str = gnome_db_gbr_get_locale_dir_path ();
+        bindtextdomain (GETTEXT_PACKAGE, str);
+	g_free (str);
+
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 
@@ -861,7 +845,7 @@ main (int argc, char **argv)
 					  "foreground", "DarkGoldenrod4",
 					  NULL);
   
-	gtk_window_set_default_size (GTK_WINDOW (window), 600, 400);
+	gtk_window_set_default_size (GTK_WINDOW (window), 600, 700);
 	gtk_widget_show_all (window);
   
 	load_file (gnomedb_demos[0].filename);
@@ -872,8 +856,10 @@ main (int argc, char **argv)
 						  _("<b><big>Note:\n</big></b>Many of the demonstrated items use an\n"
 						    "opened connection to the SQLite using the\n"
 						    "'%s' file.\n\n"
-						    "The <i>demo_cnc</i> and <i>demo_dict</i> objects are\n"
-						    "created by the framework and made available\n"
+						    "In the source code shown here, "
+						    "The <i>demo_cnc</i> and <i>demo_dict</i>\n"
+						    "objects are created by the framework and "
+						    "made available\n"
 						    "to all the demonstrated items."), full_filename);
 	g_free (full_filename);
 	g_signal_connect_swapped (msg, "response",
