@@ -20,8 +20,10 @@
 
 #include <glib/gi18n-lib.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb-extra/libgnomedb-extra.h>
 #include <libgnomedb/gnome-db-util.h>
 #include <gtk/gtk.h>
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 /* options */
 gchar *test_file = NULL;
@@ -76,6 +78,7 @@ main (int argc, char **argv)
 	GOptionContext *context;
 	GError *error = NULL;
 	GdaConnection *cnc;
+	gchar *str;
 
 	/* command line parsing */
         context = g_option_context_new ("[Data source name] - Test a data source");
@@ -87,7 +90,10 @@ main (int argc, char **argv)
         g_option_context_free (context);
 	
 	/* Initialize i18n support */
-	bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
+	str = gnome_db_gbr_get_locale_dir_path ();
+	bindtextdomain (GETTEXT_PACKAGE, str);
+	g_free (str);
+
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
 	
@@ -133,7 +139,10 @@ main (int argc, char **argv)
 	config->mainwin = mainwin;
 
 	gtk_window_set_title (GTK_WINDOW (mainwin), _("Database browser"));
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	
+	str = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	icon = gdk_pixbuf_new_from_file (str, NULL);
+	g_free (str);
 	if (icon) {
 		gtk_window_set_icon (GTK_WINDOW (mainwin), icon);
 		g_object_unref (icon);
