@@ -72,7 +72,7 @@ gnome_db_canvas_entity_get_type (void)
 {
 	static GType type = 0;
 
-        if (!type) {
+        if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbCanvasEntityClass),
 			(GBaseInitFunc) NULL,
