@@ -202,7 +202,7 @@ gnome_db_data_cell_renderer_combo_class_init (GnomeDbDataCellRendererComboClass
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GnomeDbDataCellRendererComboClass, changed),
 			      NULL, NULL,
-			      marshal_VOID__STRING_POINTER_POINTER,
+			      gnome_db_marshal_VOID__STRING_POINTER_POINTER,
 			      G_TYPE_NONE, 3,
 			      G_TYPE_STRING,
 			      G_TYPE_POINTER,
