@@ -186,7 +186,7 @@ gnome_db_data_cell_renderer_info_class_init (GnomeDbDataCellRendererInfoClass *c
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GnomeDbDataCellRendererInfoClass, status_changed),
 			      NULL, NULL,
-			      marshal_VOID__STRING_ENUM,
+			      gnome_db_marshal_VOID__STRING_ENUM,
 			      G_TYPE_NONE, 2,
 			      G_TYPE_STRING,
 			      GDA_TYPE_VALUE_ATTRIBUTE);
