@@ -51,7 +51,7 @@ struct _GnomeDbEntryBooleanClass
 	GnomeDbEntryWrapperClass         parent_class;
 };
 
-GType        gnome_db_entry_boolean_get_type        (void);
+GType        gnome_db_entry_boolean_get_type        (void) G_GNUC_CONST;
 GtkWidget   *gnome_db_entry_boolean_new             (GdaDataHandler *dh, GType type);
 
 
