@@ -1,6 +1,6 @@
 /* gnome-db-entry-common-time.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 #include <libgda/gda-data-handler.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
+#include "gnome-db-format-entry.h"
 
 /* 
  * Main static functions 
@@ -83,7 +84,7 @@ struct _GnomeDbEntryCommonTimePrivate
 	GtkWidget *hbox;
 
 	/* Last value set */
-	GValue  *last_value_set;
+	GValue    *last_value_set;
 };
 
 static void
@@ -97,7 +98,7 @@ gnome_db_entry_common_time_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbEntryCommonTimeClass),
 			(GBaseInitFunc) NULL,
@@ -148,7 +149,7 @@ gnome_db_entry_common_time_class_init (GnomeDbEntryCommonTimeClass * class)
 					 g_param_spec_boolean ("editing_cancelled", NULL, NULL, FALSE, G_PARAM_READABLE));
 	g_object_class_install_property (object_class, PROP_TYPE,
 					 g_param_spec_uint ("type", NULL, NULL, 0, G_MAXUINT, GDA_TYPE_TIME, 
-							    G_PARAM_WRITABLE));
+							    G_PARAM_WRITABLE | G_PARAM_READABLE));
 }
 
 static void
@@ -232,9 +233,9 @@ gnome_db_entry_common_time_finalize (GObject   * object)
 
 static void
 gnome_db_entry_common_time_set_property (GObject *object,
-				  guint param_id,
-				  const GValue *value,
-				  GParamSpec *pspec)
+					 guint param_id,
+					 const GValue *value,
+					 GParamSpec *pspec)
 {
 	GnomeDbEntryCommonTime *mgtim;
 
@@ -253,9 +254,9 @@ gnome_db_entry_common_time_set_property (GObject *object,
 
 static void
 gnome_db_entry_common_time_get_property (GObject *object,
-			    guint param_id,
-			    GValue *value,
-			    GParamSpec *pspec)
+					 guint param_id,
+					 GValue *value,
+					 GParamSpec *pspec)
 {
 	GnomeDbEntryCommonTime *mgtim;
 	gboolean cancelled;
@@ -272,6 +273,9 @@ gnome_db_entry_common_time_get_property (GObject *object,
 				cancelled = GTK_ENTRY (mgtim->priv->entry_time)->editing_canceled;
 			g_value_set_boolean (value, cancelled);
 			break;
+		case PROP_TYPE:
+			g_value_set_uint (value, gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (object)));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
@@ -323,7 +327,7 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 	if (type == G_TYPE_DATE) {
 		if (value) {
 			if (gda_value_is_null ((GValue *) value))
-				gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_date), "");
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date), NULL);
 			else {
 				gchar *str;
 				
@@ -333,12 +337,12 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 			}
 		}
 		else 
-			gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_date), "");
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date), NULL);
 	}
 	else if (type == GDA_TYPE_TIME) {
 		if (value) {
 			if (gda_value_is_null ((GValue *) value))
-				gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), "");
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), NULL);
 			else {
 				gchar *str;
 				
@@ -348,26 +352,26 @@ real_set_value (GnomeDbEntryWrapper *mgwrap, const GValue *value)
 			}
 		}
 		else 
-			gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), "");
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), NULL);
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
 		if (value) {
 			if (gda_value_is_null ((GValue *) value))
-				gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), "");
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), NULL);
 			else {
 				gchar *str, *ptr;
 				
 				str = gda_data_handler_get_str_from_value (dh, value);
 				ptr = strtok (str, " ");
-				gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_date), ptr);
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date), ptr);
 				ptr = strtok (NULL, " ");
-				gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), ptr);
+				gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), ptr);
 				g_free (str);
 			}
 		}
 		else {
-			gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_date), "");
-			gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), "");
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date), NULL);
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), NULL);
 		}
 	}
 	else
@@ -388,7 +392,6 @@ real_get_value (GnomeDbEntryWrapper *mgwrap)
 	GValue *value = NULL;
 	GnomeDbEntryCommonTime *mgtim;
 	GdaDataHandler *dh;
-	const gchar *str;
 	gchar *str2;
 	GType type;
 
@@ -400,12 +403,13 @@ real_get_value (GnomeDbEntryWrapper *mgwrap)
 	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
 
 	if (type == G_TYPE_DATE) {
-		str = gtk_entry_get_text (GTK_ENTRY (mgtim->priv->entry_date));
-		value = gda_data_handler_get_value_from_str (dh, str, type); /* FIXME: not SQL but STR */
+		str2 = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date));
+		value = gda_data_handler_get_value_from_str (dh, str2, type); /* FIXME: not SQL but STR */
+		g_free (str2);
 	}
 	else if (type == GDA_TYPE_TIME) {
-		str = gtk_entry_get_text (GTK_ENTRY (mgtim->priv->entry_time));
-		value = gda_data_handler_get_value_from_str (dh, str, type);
+		str2 = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time));
+		value = gda_data_handler_get_value_from_str (dh, str2, type);
 		if (mgtim->priv->last_value_set && 
 		    gda_value_isa (mgtim->priv->last_value_set, GDA_TYPE_TIME)) {
 			/* copy the 'timezone' part, we we have not modified */
@@ -416,11 +420,16 @@ real_get_value (GnomeDbEntryWrapper *mgwrap)
 			gda_value_set_time (value, gdatime);
 			g_free (gdatime);
 		}
+		g_free (str2);
 	}
 	else if (type == GDA_TYPE_TIMESTAMP) {
-		str2 = g_strdup_printf ("%s %s",
-					gtk_entry_get_text (GTK_ENTRY (mgtim->priv->entry_date)),
-					gtk_entry_get_text (GTK_ENTRY (mgtim->priv->entry_time)));
+		gchar *tmpstr, *tmpstr2;
+
+		tmpstr = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time));
+		tmpstr2 = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date));
+		str2 = g_strdup_printf ("%s %s", tmpstr2, tmpstr);
+		g_free (tmpstr);
+		g_free (tmpstr2);
 		value = gda_data_handler_get_value_from_str (dh, str2, type);
 		g_free (str2);
 		if (mgtim->priv->last_value_set && 
@@ -548,14 +557,21 @@ static GtkWidget *
 create_entry_date (GnomeDbEntryCommonTime *mgtim)
 {
 	GtkWidget *wid, *hb, *window, *arrow;
+	GdaDataHandler *dh;
 
 	/* top widget */
 	hb = gtk_hbox_new (FALSE, 3);
 
 	/* text entry */
-	wid = gtk_entry_new ();
-	gtk_entry_set_max_length (GTK_ENTRY (wid), 10); /* for a numerical date format */
-	gtk_entry_set_width_chars (GTK_ENTRY (wid), 10);
+	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgtim));
+	wid = gnome_db_format_entry_new ();
+	if (GDA_IS_HANDLER_TIME (dh)) {
+		gchar *str;
+		str = gda_handler_time_get_format (GDA_HANDLER_TIME (dh), G_TYPE_DATE);
+		gnome_db_format_entry_set_format (GNOME_DB_FORMAT_ENTRY (wid), str, NULL, NULL);
+		gtk_entry_set_width_chars (GTK_ENTRY (wid), g_utf8_strlen (str, -1));
+		g_free (str);
+	}
 	gtk_box_pack_start (GTK_BOX (hb), wid, FALSE, FALSE, 0);
 	gtk_widget_show (wid);
 	mgtim->priv->entry_date = wid;
@@ -614,16 +630,17 @@ internal_set_time (GtkWidget *widget, GnomeDbEntryCommonTime *mgtim)
 	type = gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgtim));
 	if (type == GDA_TYPE_TIMESTAMP) {
 		GdaDataHandler *dh;
-		const gchar *str;
+		gchar *str;
 		GValue *value;
 
 		dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgtim));
-		str = gtk_entry_get_text (GTK_ENTRY (mgtim->priv->entry_time));
+		str = gnome_db_format_entry_get_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time));
 		value = gda_data_handler_get_value_from_str (dh, str, type);
 		if (!value || gda_value_is_null (value)) 
-			gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_time), "00:00:00");
+			gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_time), "00:00:00");
 		if (value)
 			gda_value_free (value);
+		g_free (str);
 	}
 }
 
@@ -702,8 +719,7 @@ date_day_selected (GtkCalendar *calendar, GnomeDbEntryCommonTime *mgtim)
         buffer[sizeof(buffer)-1] = '\0';
 
         str_utf8 = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
-        gtk_entry_set_text (GTK_ENTRY (mgtim->priv->entry_date),
-                            str_utf8 ? str_utf8 : "");
+        gnome_db_format_entry_set_text (GNOME_DB_FORMAT_ENTRY (mgtim->priv->entry_date), str_utf8);
         g_free (str_utf8);
 }
 
@@ -843,20 +859,28 @@ static GtkWidget *
 create_entry_time (GnomeDbEntryCommonTime *mgtim)
 {
 	GtkWidget *wid, *hb;
+	GdaDataHandler *dh;
 
 	hb = gtk_hbox_new (FALSE, 3);
 
 	/* text entry */
-        wid = gtk_entry_new ();
+	dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgtim));
+        wid = gnome_db_format_entry_new ();
         gtk_box_pack_start (GTK_BOX (hb), wid, FALSE, FALSE, 0);
-        gtk_entry_set_max_length (GTK_ENTRY (wid), 8); /* max for HH:MM:SS format */
-	gtk_entry_set_width_chars (GTK_ENTRY (wid), 8);
+	if (GDA_IS_HANDLER_TIME (dh)) {
+		gchar *str;
+		str = gda_handler_time_get_format (GDA_HANDLER_TIME (dh), GDA_TYPE_TIME);
+		gnome_db_format_entry_set_format (GNOME_DB_FORMAT_ENTRY (wid), str, NULL, NULL);
+		gtk_entry_set_width_chars (GTK_ENTRY (wid), g_utf8_strlen (str, -1));
+		g_free (str);
+	}
         gtk_widget_show (wid);
         mgtim->priv->entry_time = wid;
 
         /* small label */
         wid = gtk_label_new (_("hh:mm:ss"));
-        gtk_box_pack_start (GTK_BOX (hb), wid, FALSE, FALSE, 0);
+	gtk_misc_set_alignment (GTK_MISC (wid), 0., 0.5);
+        gtk_box_pack_start (GTK_BOX (hb), wid, TRUE, TRUE, 0);
         gtk_widget_show (wid);
 	mgtim->priv->legend = wid;
 
