@@ -1,6 +1,6 @@
 /* gnome-db-entry-string.h
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -50,8 +50,8 @@ struct _GnomeDbEntryStringClass
 	GnomeDbEntryWrapperClass         parent_class;
 };
 
-GType        gnome_db_entry_string_get_type        (void);
-GtkWidget   *gnome_db_entry_string_new             (GdaDataHandler *dh, GType type);
+GType        gnome_db_entry_string_get_type        (void) G_GNUC_CONST;
+GtkWidget   *gnome_db_entry_string_new             (GdaDataHandler *dh, GType type, const gchar *options);
 
 
 G_END_DECLS
