@@ -240,7 +240,7 @@ gnome_db_connection_properties_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbConnectionPropertiesClass),
 			(GBaseInitFunc) NULL,
