@@ -86,7 +86,7 @@ gnome_db_data_import_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataImportClass),
 			(GBaseInitFunc) NULL,
@@ -191,12 +191,12 @@ gnome_db_data_import_init (GnomeDbDataImport * import)
 	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_SHRINK | GTK_FILL, 0, 0, 0);
 
-	encs = gda_data_model_import_new_file (DATADIR "/gnome-db/import_encodings.xml", TRUE, NULL);
+	encs = gda_data_model_import_new_file (DATADIR "/gnome-db-3.0/import_encodings.xml", TRUE, NULL);
 	encs_errors = gda_data_model_import_get_errors (GDA_DATA_MODEL_IMPORT (encs));
 	if (encs_errors) {
 		for (; encs_errors; encs_errors = g_slist_next (encs_errors)) {
 			g_print ("Error: %s\n", encs_errors->data && ((GError *) encs_errors->data)->message ?
-				 ((GError *) encs_errors->data)->message : "???");
+				 ((GError *) encs_errors->data)->message : _("no detail"));
 		}
 		import->priv->encoding_combo = NULL;
 	}
@@ -505,6 +505,7 @@ spec_changed_cb (GtkWidget *wid, GnomeDbDataImport *import)
 		g_object_set (info, "flags", 
 			      GNOME_DB_DATA_WIDGET_INFO_CHUNCK_CHANGE_BUTTONS | 
 			      GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW, NULL);
+		g_object_unref (info);
 
 		gtk_box_pack_start (GTK_BOX (import->priv->preview_box), import->priv->preview_grid, TRUE, TRUE, 0);
 		gtk_widget_show (import->priv->preview_grid);
