@@ -26,6 +26,9 @@
 #include <libgda/gda-config.h>
 #include <libgnomedb/gnome-db-data-source-selector.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <gtk/gtkcelllayout.h>
+#include <gtk/gtkcellrenderer.h>
+#include <gtk/gtkcellrenderertext.h>
 
 #define PARENT_TYPE GTK_TYPE_COMBO_BOX
 
@@ -38,6 +41,21 @@ static void gnome_db_data_source_selector_init       (GnomeDbDataSourceSelector
 						      GnomeDbDataSourceSelectorClass *klass);
 static void gnome_db_data_source_selector_finalize   (GObject *object);
 
+static void gnome_db_data_source_selector_set_property(GObject *object,
+                                                       guint param_id,
+                                                       const GValue *value,
+                                                       GParamSpec *pspec);
+static void gnome_db_data_source_selector_get_property(GObject *object,
+                                                       guint param_id,
+                                                       GValue *value,
+                                                       GParamSpec *pspec);
+
+enum {
+	PROP_0,
+
+	PROP_SOURCE_NAME
+};
+
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
 
@@ -107,12 +125,18 @@ gnome_db_data_source_selector_class_init (GnomeDbDataSourceSelectorClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gnome_db_data_source_selector_finalize;
+	object_class->set_property = gnome_db_data_source_selector_set_property;
+	object_class->get_property = gnome_db_data_source_selector_get_property;
+
+	g_object_class_install_property (object_class, PROP_SOURCE_NAME,
+	                                 g_param_spec_string ("source-name", NULL, NULL, NULL,
+	                                                      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
 config_changed_cb (const gchar *path, gpointer user_data)
 {
-	const gchar *current_dsn;
+	gchar *current_dsn;
 	GnomeDbDataSourceSelector *selector = (GnomeDbDataSourceSelector *) user_data;
 
 	g_return_if_fail (GNOME_DB_IS_DATA_SOURCE_SELECTOR (selector));
@@ -141,7 +165,7 @@ gnome_db_data_source_selector_init (GnomeDbDataSourceSelector *selector,
 		selector);
 
 	/* add textual renderer */
-	renderer = gtk_cell_renderer_text_new ();
+	renderer = GTK_CELL_RENDERER (gtk_cell_renderer_text_new ());
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (selector), renderer, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (selector), renderer,
 				       "text", 0);
@@ -167,12 +191,42 @@ gnome_db_data_source_selector_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void gnome_db_data_source_selector_set_property(GObject *object,
+                                                       guint param_id,
+                                                       const GValue *value,
+                                                       GParamSpec *pspec)
+{
+	GnomeDbDataSourceSelector *selector;
+	selector = GNOME_DB_DATA_SOURCE_SELECTOR (object);
+
+	switch (param_id) {
+	case PROP_SOURCE_NAME:
+		show_data_sources (selector, g_value_get_string (value));
+		break;
+	}
+}
+
+static void gnome_db_data_source_selector_get_property(GObject *object,
+                                                       guint param_id,
+                                                       GValue *value,
+                                                       GParamSpec *pspec)
+{
+	GnomeDbDataSourceSelector *selector;
+	selector = GNOME_DB_DATA_SOURCE_SELECTOR (object);
+
+	switch (param_id) {
+	case PROP_SOURCE_NAME:
+		g_assert_not_reached ();
+		break;
+	}
+}
+
 GType
 gnome_db_data_source_selector_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataSourceSelectorClass),
 			(GBaseInitFunc) NULL,
@@ -207,8 +261,8 @@ gnome_db_data_source_selector_new (const gchar *name)
 {
 	GnomeDbDataSourceSelector *selector;
 
-	selector = g_object_new (GNOME_DB_TYPE_DATA_SOURCE_SELECTOR, NULL);
-	show_data_sources (selector, name);
+	selector = g_object_new (GNOME_DB_TYPE_DATA_SOURCE_SELECTOR,
+	                         "name", name, NULL);
 
 	return GTK_WIDGET (selector);
 }
@@ -219,7 +273,7 @@ gnome_db_data_source_selector_new (const gchar *name)
  *
  * Get the Data Source Name (DSN) actualy selected in the #GnomeDbDataSourceSelector.
  *
- * Returns: the DSN name actualy selected.
+ * Returns: the DSN name actualy selected as a new string.
  */
 gchar *
 gnome_db_data_source_selector_get_dsn (GnomeDbDataSourceSelector *selector)
@@ -230,7 +284,6 @@ gnome_db_data_source_selector_get_dsn (GnomeDbDataSourceSelector *selector)
         gtk_combo_box_get_active_iter (GTK_COMBO_BOX (selector), &iter);
         
         gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX(selector)), &iter, 0, &dsn, -1);
-        
 
 	return dsn;
 }
