@@ -75,7 +75,7 @@ gnome_db_data_store_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataStoreClass),
 			(GBaseInitFunc) NULL,
@@ -289,7 +289,7 @@ gnome_db_data_store_get_property (GObject *object,
 		switch (param_id) {
 		case PROP_MODEL:
 			/* FIXME: the requested model can be different than the proxy as
-			   the aim of this property is to retreive the data model given to
+			   the aim of this property is to retrieve the data model given to
 			   the new() function (which is the same as the one given to the PROP_MODEL
 			   property) */
                 case PROP_PROXY:
@@ -542,7 +542,7 @@ gnome_db_data_store_get_iter_from_values (GnomeDbDataStore *store, GtkTreeIter *
 /* 
  * GtkTreeModel Interface implementation
  *
- * REM about the GtkTreeIter: only the iter->user_data is used to retreive a row in the data model:
+ * REM about the GtkTreeIter: only the iter->user_data is used to retrieve a row in the data model:
  *     iter->user_data contains the requested row number in the GtkTreeModel numbers
  *     iter->stamp is reset any time the model changes.
  */
