@@ -80,7 +80,7 @@ gnome_db_data_widget_info_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDataWidgetInfoClass),
 			(GBaseInitFunc) NULL,
