@@ -54,6 +54,21 @@ static void gnome_db_dsn_spec_init       (GnomeDbDsnSpec *spec,
 static void gnome_db_dsn_spec_finalize   (GObject *object);
 static void gnome_db_dsn_spec_dispose    (GObject *object);
 
+static void gnome_db_dsn_spec_set_property (GObject *object,
+                                            guint param_id,
+                                            const GValue *value,
+                                            GParamSpec *pspec);
+static void gnome_db_dsn_spec_get_property (GObject *object,
+                                            guint param_id,
+                                            GValue *value,
+                                            GParamSpec *pspec);
+
+enum {
+	PROP_0,
+
+	PROP_PROVIDER
+};
+
 enum {
 	CHANGED,
 	LAST_SIGNAL
@@ -76,8 +91,14 @@ gnome_db_dsn_spec_class_init (GnomeDbDsnSpecClass *klass)
 
 	object_class->dispose = gnome_db_dsn_spec_dispose;
 	object_class->finalize = gnome_db_dsn_spec_finalize;
+	object_class->set_property = gnome_db_dsn_spec_set_property;
+	object_class->get_property = gnome_db_dsn_spec_get_property;
 	klass->changed = NULL;
 
+	g_object_class_install_property (object_class, PROP_PROVIDER,
+	                                 g_param_spec_string ("provider", NULL, NULL, NULL,
+					                      G_PARAM_READWRITE));
+
 	/* add class signals */
 	gnome_db_dsn_spec_signals[CHANGED] =
 		g_signal_new ("changed",
@@ -297,12 +318,43 @@ gnome_db_dsn_spec_finalize (GObject *object)
 	parent_class->finalize (object);
 }
 
+static void gnome_db_dsn_spec_set_property (GObject *object,
+                                            guint param_id,
+                                            const GValue *value,
+                                            GParamSpec *pspec)
+{
+	GnomeDbDsnSpec *spec;
+	spec = GNOME_DB_DSN_SPEC (object);
+
+	switch(param_id) {
+	case PROP_PROVIDER:
+		gnome_db_dsn_spec_set_provider (spec,
+		                                g_value_get_string (value));
+		break;
+	}
+}
+
+static void gnome_db_dsn_spec_get_property (GObject *object,
+                                            guint param_id,
+                                            GValue *value,
+                                            GParamSpec *pspec)
+{
+	GnomeDbDsnSpec *spec;
+	spec = GNOME_DB_DSN_SPEC (object);
+
+	switch (param_id) {
+	case PROP_PROVIDER:
+		g_value_set_string (value, spec->priv->provider);
+		break;
+	}
+}
+
 GType
 gnome_db_dsn_spec_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbDsnSpecClass),
 			(GBaseInitFunc) NULL,
@@ -333,10 +385,8 @@ gnome_db_dsn_spec_new (const gchar *provider)
 {
 	GnomeDbDsnSpec *spec;
 
-	spec = g_object_new (GNOME_DB_TYPE_DSN_SPEC, NULL);
-	if (provider)
-		spec->priv->provider = g_strdup (provider);
-	adapt_form_widget (spec);
+	spec = g_object_new (GNOME_DB_TYPE_DSN_SPEC,
+	                     "provider", provider, NULL);
 
 	return GTK_WIDGET (spec);
 }
