@@ -31,6 +31,7 @@
 #include <gconf/gconf-client.h>
 #endif
 #include <libgda/graph/gda-dict-reg-graphs.h>
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 
 /* plugins list */
 GHashTable *gnome_db_plugins_hash = NULL; /* key = plugin name, value = GnomeDbPlugin structure pointer */
@@ -50,13 +51,18 @@ void
 gnome_db_init (const gchar *app_id, const gchar *version, gint nargs, gchar *args[])
 {
 	static gboolean initialized = FALSE;
+	gchar *str;
 
 	if (initialized) {
 		gda_log_error (_("Attempt to initialize an already initialized client"));
 		return;
 	}
 
-	bindtextdomain (GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
+	gnome_db_gbr_init ();
+	str = gnome_db_gbr_get_locale_dir_path ();
+	bindtextdomain (GETTEXT_PACKAGE, str);
+	g_free (str);
+
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	gda_init (app_id, version, nargs, args);
