@@ -31,6 +31,7 @@
 #include <libgnomedb/gnome-db-login-dialog.h>
 #include <libgnomedb/gnome-db-stock.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <libgnomedb/binreloc/gnome-db-binreloc.h>
 #include <glib/gi18n-lib.h>
 
 extern void gnome_db_stock_init (void);
@@ -97,6 +98,7 @@ gnome_db_login_dialog_init (GnomeDbLoginDialog *dialog, GnomeDbLoginDialogClass
 	GtkWidget *hbox, *vbox, *image, *label;
 	GtkWidget *nb;
 	gchar *str;
+	GdkPixbuf *icon;
 
 	g_return_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog));
 
@@ -150,6 +152,15 @@ gnome_db_login_dialog_init (GnomeDbLoginDialog *dialog, GnomeDbLoginDialogClass
 	gnome_db_login_set_enable_create_button (GNOME_DB_LOGIN (dialog->priv->login), TRUE);
 	gtk_widget_show (dialog->priv->login);
 	gtk_box_pack_start (GTK_BOX (vbox), dialog->priv->login, TRUE, TRUE, 0);
+
+	str = gnome_db_gbr_get_icon_path ("gnome-db.png");
+	icon = gdk_pixbuf_new_from_file (str, NULL);
+	g_free (str);
+	if (icon) {
+		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+		g_object_unref (icon);
+	}
+
 }
 
 static void
@@ -220,7 +231,7 @@ gnome_db_login_dialog_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbLoginDialogClass),
 			(GBaseInitFunc) NULL,
@@ -248,17 +259,10 @@ gnome_db_login_dialog_get_type (void)
 GtkWidget *
 gnome_db_login_dialog_new (const gchar *title)
 {
-	GdkPixbuf *icon;
 	GnomeDbLoginDialog *dialog;
 
 	dialog = g_object_new (GNOME_DB_TYPE_LOGIN_DIALOG, "title", title, NULL);
 
-	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
-	if (icon) {
-		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
-		g_object_unref (icon);
-	}
-
 	return GTK_WIDGET (dialog);
 }
 
@@ -332,10 +336,10 @@ const gchar *gnome_db_login_dialog_get_password (GnomeDbLoginDialog *dialog)
  *
  * Returns: the login widget contained in the dialog.
  */
-GtkWidget *
+GnomeDbLogin *
 gnome_db_login_dialog_get_login_widget (GnomeDbLoginDialog *dialog)
 {
 	g_return_val_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog), NULL);
-	return dialog->priv->login;
+	return GNOME_DB_LOGIN (dialog->priv->login);
 }
 
