@@ -265,7 +265,7 @@ gnome_db_login_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbLoginClass),
 			(GBaseInitFunc) NULL,
