@@ -47,7 +47,7 @@ struct _GnomeDbLoginClass {
 	GtkVBoxClass parent_class;
 };
 
-GType        gnome_db_login_get_type                 (void);
+GType        gnome_db_login_get_type                 (void) G_GNUC_CONST;
 GtkWidget   *gnome_db_login_new                      (const gchar *dsn);
 const gchar *gnome_db_login_get_dsn                  (GnomeDbLogin *login);
 void         gnome_db_login_set_dsn                  (GnomeDbLogin *login, const gchar *dsn);
