@@ -1,6 +1,6 @@
 /* gnome-db-raw-form.c
  *
- * Copyright (C) 2002 - 2006 Vivien Malerba
+ * Copyright (C) 2002 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,10 +20,12 @@
 
 #include <string.h>
 #include <libgda/libgda.h>
+#include <libgda/gda-data-proxy.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-raw-form.h"
 #include "gnome-db-data-widget.h"
 #include "gnome-db-basic-form.h"
+#include "utility.h"
 
 static void gnome_db_raw_form_class_init (GnomeDbRawFormClass * class);
 static void gnome_db_raw_form_init (GnomeDbRawForm *wid);
@@ -43,12 +45,9 @@ static void gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layou
 static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
 static void proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
+static void proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
 static void proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm *form);
 
-/* errors reporting */
-static GtkWidget *create_data_error_dialog (GnomeDbRawForm *form, gboolean with_question, GError *filled_error);
-static gboolean display_error_with_keep_or_discard_choice (GnomeDbRawForm *form, GError *filled_error);
-
 /* GnomeDbDataWidget interface */
 static void            gnome_db_raw_form_widget_init         (GnomeDbDataWidgetIface *iface);
 static GdaDataProxy   *gnome_db_raw_form_get_proxy           (GnomeDbDataWidget *iface);
@@ -91,7 +90,7 @@ gnome_db_raw_form_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbRawFormClass),
 			(GBaseInitFunc) NULL,
@@ -198,7 +197,8 @@ form_activated_cb (GnomeDbRawForm *form, gpointer data)
 				GError *error = NULL;
 				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
 					gboolean discard;
-					discard = display_error_with_keep_or_discard_choice (form, error);
+					discard = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) form, 
+												     error);
 					if (discard)
 						gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
 					g_error_free (error);
@@ -217,8 +217,14 @@ form_param_changed_cb (GnomeDbRawForm *form, gpointer data)
 		row = gda_data_model_iter_get_row (form->priv->iter);
 		if (row >= 0) {
 			/* write back the current row */
-			if (gda_data_proxy_row_has_changed (form->priv->proxy, row))
-				gda_data_proxy_apply_row_changes (form->priv->proxy, row, NULL);
+			if (gda_data_proxy_row_has_changed (form->priv->proxy, row)) {
+				GError *error = NULL;
+				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+					gnome_db_utility_display_error ((GnomeDbDataWidget *) form, TRUE, error);
+					if (error)
+						g_error_free (error);
+				}
+			}
 		}
 	}
 }
@@ -291,6 +297,8 @@ gnome_db_raw_form_dispose (GObject *object)
 							      G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
 							      G_CALLBACK (proxy_changed_cb), form);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+							      G_CALLBACK (proxy_reset_cb), form);
 			g_object_unref (form->priv->proxy);
 			form->priv->proxy = NULL;
 		}
@@ -339,6 +347,8 @@ gnome_db_raw_form_set_property (GObject *object,
 								      form);
 				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
 								      G_CALLBACK (proxy_changed_cb), form);
+				g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+								      G_CALLBACK (proxy_reset_cb), form);
 				g_object_unref (G_OBJECT (form->priv->proxy));
 				form->priv->proxy = NULL;
 				form->priv->model = NULL;
@@ -368,6 +378,8 @@ gnome_db_raw_form_set_property (GObject *object,
 						  G_CALLBACK (proxy_row_inserted_or_removed_cb), form);
 				g_signal_connect (G_OBJECT (form->priv->proxy), "changed",
 						  G_CALLBACK (proxy_changed_cb), form);
+				g_signal_connect (G_OBJECT (form->priv->proxy), "reset",
+						  G_CALLBACK (proxy_reset_cb), form);
 
 				/* we don't want chuncking */
 				g_object_set (object, "paramlist", form->priv->iter, NULL);
@@ -377,6 +389,7 @@ gnome_db_raw_form_set_property (GObject *object,
 
 			gnome_db_raw_form_widget_set_write_mode ((GnomeDbDataWidget *) form, form->priv->write_mode);
 			g_signal_emit_by_name (object, "proxy_changed", form->priv->proxy);
+			g_signal_emit_by_name (object, "iter_changed", form->priv->iter);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -415,8 +428,8 @@ iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 			if (gda_data_proxy_row_has_changed (form->priv->proxy, row)) {
 				GError *error = NULL;
 				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
-					allow_change = display_error_with_keep_or_discard_choice (form, 
-												  error);
+					allow_change = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) form, 
+													  error);
 					if (allow_change)
 						gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
 					g_error_free (error);
@@ -441,13 +454,17 @@ iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 
 		params = ((GdaParameterList *) iter)->parameters;
 		while (params) {
+			gboolean explicit_hide = FALSE;
 			param = (GdaParameter *) params->data;
 			attributes = gda_data_proxy_get_value_attributes (form->priv->proxy, row, 
 							       gda_data_model_iter_get_column_for_param (iter, param));
 			gnome_db_basic_form_entry_set_editable ((GnomeDbBasicForm *) form, 
 								param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
-			gnome_db_basic_form_entry_show ((GnomeDbBasicForm *) form, 
-							param, !(attributes & GDA_VALUE_ATTR_UNUSED));
+			if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "_gnome_db_explicit_show")) == 1)
+				explicit_hide = TRUE;
+			if (!explicit_hide) 
+				gnome_db_basic_form_entry_show ((GnomeDbBasicForm *) form, 
+								param, !(attributes & GDA_VALUE_ATTR_UNUSED));
 			params = params->next;
 		}
 	}
@@ -461,6 +478,14 @@ proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form)
 				  gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) == 0 ? FALSE : TRUE);
 }
 
+static void
+proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawForm *form)
+{
+	g_object_ref (G_OBJECT (proxy));
+	g_object_set (G_OBJECT (form), "model", proxy, NULL);
+	g_object_unref (G_OBJECT (proxy));
+}
+
 static void
 proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm *form)
 {
@@ -517,9 +542,11 @@ gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTabl
 		node = gda_parameter_list_find_node_for_param (GDA_PARAMETER_LIST (form->priv->iter), 
 							       GDA_PARAMETER (list->data));
 		g_assert (node);
-		if (node->hint & GDA_PARAMETER_LIST_PARAM_HIDE)
+		if (node->hint & GDA_PARAMETER_LIST_PARAM_HIDE) {
 			gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form),
 							GDA_PARAMETER (list->data), FALSE);
+			g_object_set_data (G_OBJECT (list->data), "_gnome_db_explicit_show", GINT_TO_POINTER (1));
+		}
 		list = g_slist_next (list);
 	}
 
@@ -587,8 +614,14 @@ action_delete_cb (GtkAction *action, GnomeDbRawForm *form)
 		gint newrow;
 
 		newrow = gda_data_model_iter_get_row (form->priv->iter);
-		if (row == newrow) /* => row has been marked as delete but nit yet really deleted */
-			gda_data_proxy_apply_row_changes (form->priv->proxy, row, NULL);
+		if (row == newrow) {/* => row has been marked as delete but nit yet really deleted */
+			GError *error = NULL;
+			if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+				gnome_db_utility_display_error ((GnomeDbDataWidget *) form, TRUE, error);
+				if (error)
+					g_error_free (error);
+			}
+		}
 	}
 }
 
@@ -606,21 +639,17 @@ static void
 action_commit_cb (GtkAction *action, GnomeDbRawForm *form)
 {
 	gint row;
+	GError *error = NULL;
+	gboolean allok = TRUE;
+	gint mod1, mod2;
 
+	mod1 = gda_data_proxy_get_n_modified_rows (form->priv->proxy);
 	row = gda_data_model_iter_get_row (form->priv->iter);
 	if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
 		gint newrow;
-		GError *error = NULL;
 
-		if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
-			GtkWidget *dlg;
-
-			dlg = create_data_error_dialog (form, FALSE, error);
-			gtk_dialog_run (GTK_DIALOG (dlg));
-			gtk_widget_destroy (dlg);
-			g_error_free (error);
-		}
-		else {
+		allok = gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error);
+		if (allok) {
 			newrow = gda_data_model_iter_get_row (form->priv->iter);
 			if (row != newrow) /* => current row has changed because the proxy had to emit a "row_removed" when
 					      actually succedded the commit 
@@ -630,7 +659,19 @@ action_commit_cb (GtkAction *action, GnomeDbRawForm *form)
 		}
 	}
 	else
-		gda_data_proxy_apply_all_changes (form->priv->proxy, NULL);
+		allok = gda_data_proxy_apply_all_changes (form->priv->proxy, &error);
+
+	mod2 = gda_data_proxy_get_n_modified_rows (form->priv->proxy);
+	if (!allok) {
+		if (mod1 != mod2)
+			/* the data model has changed while doing the writing */
+			gnome_db_utility_display_error ((GnomeDbDataWidget *) form, FALSE, error);
+		else
+			gnome_db_utility_display_error ((GnomeDbDataWidget *) form, TRUE, error);
+		g_error_free (error);
+	}
+
+	iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
 }
 
 static void
@@ -732,6 +773,7 @@ gnome_db_raw_form_col_set_show (GnomeDbDataWidget *iface, gint column, gboolean
 	param = gda_data_model_iter_get_param_for_column (form->priv->iter, column);
 	g_return_if_fail (param);
 	gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form), param, shown);
+	g_object_set_data (G_OBJECT (param), "_gnome_db_explicit_show", GINT_TO_POINTER (shown ? 2 : 1));
 }
 
 void
@@ -838,48 +880,3 @@ gnome_db_raw_form_widget_get_write_mode (GnomeDbDataWidget *iface)
 
 	return form->priv->write_mode;
 }
-
-/*
- * Errors reporting
- */
-static GtkWidget *
-create_data_error_dialog (GnomeDbRawForm *form, gboolean with_question, GError *filled_error)
-{
-	GtkWidget *dlg;
-	gchar *msg;
-
-	if (with_question)
-		msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
-				       _("Current modified data is invalid"),
-				       _("You may now choose to correct it, or to discard\n"
-					 "the modifications.\n\n"
-					 "What do you wan to do?"));
-	else
-		msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
-				       _("Current modified data is invalid"),
-				       _("please correct it and try again, or discard\n"
-					 "the modifications."));
-	dlg = gtk_message_dialog_new_with_markup ((GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (form)), GTK_DIALOG_MODAL,
-						  GTK_MESSAGE_ERROR, 
-						  with_question ? GTK_BUTTONS_NONE : GTK_BUTTONS_CLOSE,
-						  msg);
-	g_free (msg);
-	return dlg;
-}
-
-/*
- * Returns: TRUE if current data can be discarded
- */
-static gboolean
-display_error_with_keep_or_discard_choice (GnomeDbRawForm *form, GError *filled_error)
-{
-	GtkWidget *dlg;
-	gint res;
-	dlg = create_data_error_dialog (form, TRUE, filled_error);
-	gtk_dialog_add_buttons (GTK_DIALOG (dlg),
-				_("Discard modified data"), GTK_RESPONSE_REJECT,
-				_("Correct data first"), GTK_RESPONSE_NONE, NULL);
-	res = gtk_dialog_run (GTK_DIALOG (dlg));
-	gtk_widget_destroy (dlg);
-	return (res == GTK_RESPONSE_REJECT) ? TRUE : FALSE;
-}
