@@ -169,7 +169,7 @@ gnome_db_raw_grid_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbRawGridClass),
 			(GBaseInitFunc) NULL,
@@ -223,7 +223,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, selection_changed),
                               NULL, NULL,
-                              marshal_VOID__BOOLEAN, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__BOOLEAN, G_TYPE_NONE,
                               1, G_TYPE_BOOLEAN);
 	gnome_db_raw_grid_signals[DOUBLE_CLICKED] = 
 		g_signal_new ("double_clicked",
@@ -231,7 +231,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, double_clicked),
                               NULL, NULL,
-                              marshal_VOID__INT, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__INT, G_TYPE_NONE,
                               1, G_TYPE_INT);
 	gnome_db_raw_grid_signals[POPULATE_POPUP] = 
 		g_signal_new ("populate_popup",
@@ -239,7 +239,7 @@ gnome_db_raw_grid_class_init (GnomeDbRawGridClass * class)
                               G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbRawGridClass, populate_popup),
                               NULL, NULL,
-                              marshal_VOID__OBJECT, G_TYPE_NONE,
+                              gnome_db_marshal_VOID__OBJECT, G_TYPE_NONE,
                               1, GTK_TYPE_MENU);
 
 	object_class->dispose = gnome_db_raw_grid_dispose;
@@ -355,7 +355,7 @@ gnome_db_raw_grid_set_property (GObject *object,
                 switch (param_id) {
 		case PROP_MODEL:
 			{
-				GdaDataModel *model = GDA_DATA_MODEL(g_value_get_object (value));
+				GdaDataModel *model = GDA_DATA_MODEL (g_value_get_object (value));
 				if (model)
 					g_return_if_fail (GDA_IS_DATA_MODEL (model));
 	
@@ -368,7 +368,7 @@ gnome_db_raw_grid_set_property (GObject *object,
 				grid->priv->data_model = gda_data_proxy_get_proxied_model (grid->priv->proxy);
 				
 				gda_object_connect_destroy (grid->priv->proxy,
-								G_CALLBACK (proxy_destroyed_cb), grid);
+							    G_CALLBACK (proxy_destroyed_cb), grid);
 				g_signal_connect (grid->priv->proxy, "sample_changed",
 						  G_CALLBACK (proxy_sample_changed_cb), grid);
 				g_signal_connect (grid->priv->proxy, "row_updated",
@@ -437,7 +437,7 @@ gnome_db_raw_grid_get_property (GObject *object,
         if (grid->priv) {
                 switch (param_id) {
 		case PROP_MODEL:
-			g_value_set_pointer (value, grid->priv->data_model);
+			g_value_set_object (value, grid->priv->data_model);
 			break;
 		case PROP_INFO_CELL_VISIBLE:
 			g_value_set_boolean(value, grid->priv->default_show_info_cell);
