@@ -66,7 +66,9 @@ struct _GnomeDbServerOperationPriv
 	GdaServerOperation     *op;
 	GSList                 *widget_data; /* list of WidgetData structures */
 	GtkTooltips            *tooltips;
+#ifdef HAVE_LIBGLADE
 	GladeXML               *glade;
+#endif
 	gboolean                opt_header;
 };
 
@@ -159,7 +161,7 @@ gnome_db_server_operation_get_type (void)
 {
 	static GType type = 0;
 
-	if (!type) {
+	if (G_UNLIKELY (type == 0)) {
 		static const GTypeInfo info = {
 			sizeof (GnomeDbServerOperationClass),
 			(GBaseInitFunc) NULL,
@@ -212,7 +214,9 @@ gnome_db_server_operation_init (GnomeDbServerOperation * wid)
 	wid->priv->tooltips = gtk_tooltips_new ();
 	g_object_ref (G_OBJECT (wid->priv->tooltips));
 	gtk_object_sink (GTK_OBJECT (wid->priv->tooltips));
+#ifdef HAVE_LIBGLADE
 	wid->priv->glade = NULL;
+#endif
 	wid->priv->opt_header = FALSE;
 }
 
@@ -267,8 +271,10 @@ gnome_db_server_operation_dispose (GObject *object)
 		}
 		
 		g_object_unref (G_OBJECT (form->priv->tooltips));
+#ifdef HAVE_LIBGLADE
 		if (form->priv->glade)
 			g_object_unref (form->priv->glade);
+#endif
 
 		/* the private area itself */
 		g_free (form->priv);
@@ -471,7 +477,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		g_object_set (G_OBJECT (grid), "info-cell-visible", FALSE, NULL);
 
 		winfo = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (grid), 
-						       GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS);
+						       GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS);
 
 		box = gtk_vbox_new (FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (box), plwid, TRUE, TRUE, 0);
@@ -712,6 +718,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		return;
 
 	/* load Glade file for specific GUI if it exists */
+#ifdef HAVE_LIBGLADE
 	glade_file = g_build_filename (DATADIR, "gnome-db", "server_operation.glade", NULL);
 	form->priv->glade = glade_xml_new (glade_file, 
 					   gda_server_operation_op_type_to_string (gda_server_operation_get_op_type (form->priv->op)), 
@@ -730,6 +737,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 			form->priv->glade = NULL;
 		}
 	}
+#endif
 
 	/* user visible widgets */
 	topnodes = gda_server_operation_get_root_nodes (form->priv->op);
@@ -739,6 +747,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		gchar *section_str;
 		GtkWidget *container = NULL;
 
+#ifdef HAVE_LIBGLADE
 		if (form->priv->glade) {
 			container = glade_xml_get_widget (form->priv->glade, topnodes[i]);
 			if (!container) {
@@ -746,6 +755,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 				continue;
 			}
 		}
+#endif
 		if (!container)
 			container = (GtkWidget *) form;
 
@@ -820,6 +830,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 	}
 
 	/* destroying unused widgets in the Glade description */
+#ifdef HAVE_LIBGLADE
 	if (form->priv->glade) {
 		GList *widgets, *list;
 
@@ -850,6 +861,7 @@ gnome_db_server_operation_fill (GnomeDbServerOperation *form)
 		}
 		g_list_free (widgets);
 	}
+#endif
 
 	g_strfreev (topnodes);
 	
@@ -1172,7 +1184,7 @@ create_table_fields_array_create_widget (GnomeDbServerOperation *form, const gch
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
 
-	grid_fields = gnome_db_raw_grid_new (proxy);
+	grid_fields = gnome_db_raw_grid_new (GDA_DATA_MODEL(proxy));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (grid_fields), FALSE);
 	gtk_widget_set_usize (grid_fields, 300, 200);
 	g_object_set (G_OBJECT (grid_fields), "info-cell-visible", FALSE, NULL);	
@@ -1193,7 +1205,7 @@ create_table_fields_array_create_widget (GnomeDbServerOperation *form, const gch
 
 	/* buttons to add/remove fields */
 	winfo = gnome_db_data_widget_info_new (GNOME_DB_DATA_WIDGET (form_props), 
-					       GNOME_DB_DATA_WIDGET_INFO_ROW_MODIF_BUTTONS);
+					       GNOME_DB_DATA_WIDGET_INFO_ROW_MODIFY_BUTTONS);
 
 	gtk_box_pack_start (GTK_BOX (box), winfo, FALSE, FALSE, 0);
 
