@@ -1,6 +1,6 @@
 /* utility.c
  *
- * Copyright (C) 2003 - 2006 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2003 - 2007 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 #include "gnome-db-decl.h"
 
 /**
- * utility_entry_build_actions_menu
+ * gnome_db_utility_entry_build_actions_menu
  * @obj_data:
  * @attrs:
  * @function:
@@ -42,7 +42,7 @@
  * Returns: the new menu
  */
 GtkWidget *
-utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallback function)
+gnome_db_utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallback function)
 {
 	GtkWidget *menu, *mitem;
 	gchar *str;
@@ -118,7 +118,7 @@ utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallback func
 
 
 /**
- * utility_entry_build_info_colors_array
+ * gnome_db_utility_entry_build_info_colors_array
  * 
  * Creates an array of colors for the different states of an entry:
  *    Valid   <-> No special color
@@ -130,7 +130,7 @@ utility_entry_build_actions_menu (GObject *obj_data, guint attrs, GCallback func
  *
  * Returns: a new array of 6 colors
  */
-GdkColor **utility_entry_build_info_colors_array ()
+GdkColor **gnome_db_utility_entry_build_info_colors_array ()
 {
 	GdkColor **colors;
 	GdkColor *color;
@@ -194,7 +194,19 @@ GdkColor **utility_entry_build_info_colors_array ()
 }
 
 /**
- * utility_proxy_compute_attributes_for_group
+ * gnome_db_utility_markup_title
+ */
+gchar *
+gnome_db_utility_markup_title (const gchar *title, gboolean optional)
+{
+	if (!optional)
+		return g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>:", title);
+	else
+		return g_strdup_printf ("%s:", title);
+}
+
+/**
+ * gnome_db_utility_proxy_compute_attributes_for_group
  *
  * Computes an attributes from the individual attributes of the values stored in @store and
  * corresponding to the columns for the parameters in @group (as described by @model_iter), 
@@ -203,7 +215,7 @@ GdkColor **utility_entry_build_info_colors_array ()
  * Returns: the attributes
  */
 guint
-utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeDbDataStore *store, 
+gnome_db_utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeDbDataStore *store, 
 					    GdaDataModelIter *model_iter, GtkTreeIter *tree_iter, 
 					    gboolean *to_be_deleted)
 {
@@ -244,7 +256,7 @@ utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeD
 }
 
 /**
- * utility_proxy_compute_values_for_group
+ * gnome_db_utility_proxy_compute_values_for_group
  * Computes a list of values containing the individual values stored in @store and
  * corresponding to the columns for the parameters if @group, at row pointed by @tree_iter.
  * 
@@ -258,7 +270,7 @@ utility_proxy_compute_attributes_for_group (GdaParameterListGroup *group, GnomeD
  * freed.
  */
 GList *
-utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDataStore *store, 
+gnome_db_utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDataStore *store, 
 					GdaDataModelIter *model_iter, 
 					GtkTreeIter *tree_iter, gboolean model_values)
 {
@@ -378,297 +390,125 @@ utility_proxy_compute_values_for_group (GdaParameterListGroup *group, GnomeDbDat
 }
 
 
-/**
- * utility_table_field_attrs_stringify
+
+/*
+ * Errors reporting
  */
-gchar *
-utility_table_field_attrs_stringify (guint attributes)
+static GtkWidget *
+create_data_error_dialog (GnomeDbDataWidget *form, gboolean with_question, gboolean can_discard, GError *filled_error)
 {
-	if (attributes & FIELD_AUTO_INCREMENT)
-		return g_strdup ("AUTO_INCREMENT");
+	GtkWidget *dlg;
+	gchar *msg;
 
-	return NULL;
-}
-
-/**
- * utility_table_field_attrs_parse
- */
-guint
-utility_table_field_attrs_parse (const gchar *str)
-{
-	if (!strcmp (str, "AUTO_INCREMENT"))
-		return FIELD_AUTO_INCREMENT;
-
-	return 0;
-}
-
-
-/**
- * utility_build_encoded_id
- *
- * Creates a BASE64 kind encoded string. It's not really a BASE64 because:
- * - the characters + and / of BASE64 are replaced with - and _
- * - no padding is done using the = character
- *
- * The created string is a valid NCName XML token.
- */
-static inline unsigned char
-to_uchar (gchar ch)
-{
-	return ch;
-}
-
-gchar *
-utility_build_encoded_id (const gchar *prefix, const gchar *id)
-{
-	const gchar conv[64] = 
-		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
-	gchar *str, *out;
-	const gchar *in;
-	int ln = 0, pln = 0;
-	int offset;
-
-	/* size computation */
-	if (prefix) {
-		pln = strlen (prefix);
-		ln = pln;
+	if (can_discard) {
+		if (with_question)
+			msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
+					       _("Current modified data is invalid"),
+					       _("You may now choose to correct it, or to discard\n"
+						 "the modifications.\n\n"
+						 "What do you wan to do?"));
+		else
+			msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
+					       _("Current modified data is invalid"),
+					       _("please correct it and try again, or discard\n"
+						 "the modifications."));
 	}
-
-	ln += (strlen (id) * 4 + 2) / 3 + 1;
-	str = g_new0 (char, ln);
-
-	/* copy prefix */
-	out = str;
-	if (prefix) {
-		strcpy (str, prefix);
-		out += pln;
+	else {
+		if (with_question)
+			g_warning ("Incoherence problem...\n");
+		else
+			msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
+					       _("Part of the current modified data was invalid"),
+					       _("As no transaction was used, only a part of the valid data\n"
+						 "has been written, and the remaining modification have been discarded."));
 	}
+	dlg = gtk_message_dialog_new_with_markup ((GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (form)), GTK_DIALOG_MODAL,
+						  GTK_MESSAGE_ERROR, 
+						  with_question ? GTK_BUTTONS_NONE : GTK_BUTTONS_CLOSE,
+						  msg);
+	g_free (msg);
 
-	/* create data */
-	offset = 4;
-	for (in = id; offset == 4; in += 3) {
-		offset = 0;
+	if (filled_error && filled_error->message) {
+		GtkWidget *exp, *sw, *view;
+		GtkTextBuffer *buffer;
+		GtkTextIter start, end;
+		GtkTextTag *tag;
 		
-		if (in[0]) {
-			offset = 2;
-			
-			out[0] = conv [to_uchar (in[0]) >> 2];
+		exp = gtk_expander_new (_("Error details"));
 
-			if (in[1]) {
-				offset = 3;
-				out[1] = conv [((to_uchar (in[0]) << 4) + 
-						(to_uchar (in[1]) >> 4)) & 0x3f];
+		sw = gtk_scrolled_window_new (NULL, NULL);
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+						GTK_POLICY_AUTOMATIC,
+						GTK_POLICY_AUTOMATIC);
 
-				if (in[2]) {
-					offset = 4;
+		view = gtk_text_view_new ();
+		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
+		gtk_text_buffer_set_text (buffer, filled_error->message, -1);
+						
+		/* Use a tag to change the color for just one part of the widget */
+		tag = gtk_text_buffer_create_tag (buffer, "monospace",
+						  "family", "monospace", NULL);
+		gtk_text_buffer_get_bounds (buffer, &start, &end);
+		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);
+		
+		gtk_container_add (GTK_CONTAINER (sw), view);
+		gtk_container_add (GTK_CONTAINER (exp), sw);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), exp, TRUE, TRUE, 0);
+		gtk_widget_show_all (exp);
 
-					out[2] = conv [((to_uchar (in[1]) << 2) + 
-							(to_uchar (in[2]) >> 6)) & 0x3f];
-					out[3] = conv [to_uchar (in[2]) & 0x3f];
-				}
-				else
-					out[2] = conv [(to_uchar (in[1]) << 2) & 0x3f];
-			}
-			else
-				out[1] = conv [(to_uchar (in[0]) << 4) & 0x3f];
-
-
-			out += offset;
-		}
-        }
-
-	return str;
-}
-
-#define B64(x)	     \
-  ((x) == 'A' ? 0    \
-   : (x) == 'B' ? 1  \
-   : (x) == 'C' ? 2  \
-   : (x) == 'D' ? 3  \
-   : (x) == 'E' ? 4  \
-   : (x) == 'F' ? 5  \
-   : (x) == 'G' ? 6  \
-   : (x) == 'H' ? 7  \
-   : (x) == 'I' ? 8  \
-   : (x) == 'J' ? 9  \
-   : (x) == 'K' ? 10 \
-   : (x) == 'L' ? 11 \
-   : (x) == 'M' ? 12 \
-   : (x) == 'N' ? 13 \
-   : (x) == 'O' ? 14 \
-   : (x) == 'P' ? 15 \
-   : (x) == 'Q' ? 16 \
-   : (x) == 'R' ? 17 \
-   : (x) == 'S' ? 18 \
-   : (x) == 'T' ? 19 \
-   : (x) == 'U' ? 20 \
-   : (x) == 'V' ? 21 \
-   : (x) == 'W' ? 22 \
-   : (x) == 'X' ? 23 \
-   : (x) == 'Y' ? 24 \
-   : (x) == 'Z' ? 25 \
-   : (x) == 'a' ? 26 \
-   : (x) == 'b' ? 27 \
-   : (x) == 'c' ? 28 \
-   : (x) == 'd' ? 29 \
-   : (x) == 'e' ? 30 \
-   : (x) == 'f' ? 31 \
-   : (x) == 'g' ? 32 \
-   : (x) == 'h' ? 33 \
-   : (x) == 'i' ? 34 \
-   : (x) == 'j' ? 35 \
-   : (x) == 'k' ? 36 \
-   : (x) == 'l' ? 37 \
-   : (x) == 'm' ? 38 \
-   : (x) == 'n' ? 39 \
-   : (x) == 'o' ? 40 \
-   : (x) == 'p' ? 41 \
-   : (x) == 'q' ? 42 \
-   : (x) == 'r' ? 43 \
-   : (x) == 's' ? 44 \
-   : (x) == 't' ? 45 \
-   : (x) == 'u' ? 46 \
-   : (x) == 'v' ? 47 \
-   : (x) == 'w' ? 48 \
-   : (x) == 'x' ? 49 \
-   : (x) == 'y' ? 50 \
-   : (x) == 'z' ? 51 \
-   : (x) == '0' ? 52 \
-   : (x) == '1' ? 53 \
-   : (x) == '2' ? 54 \
-   : (x) == '3' ? 55 \
-   : (x) == '4' ? 56 \
-   : (x) == '5' ? 57 \
-   : (x) == '6' ? 58 \
-   : (x) == '7' ? 59 \
-   : (x) == '8' ? 60 \
-   : (x) == '9' ? 61 \
-   : (x) == '-' ? 62 \
-   : (x) == '_' ? 63 \
-   : -1)
-
-static const signed char b64[0x100] = {
-  B64 (0), B64 (1), B64 (2), B64 (3),
-  B64 (4), B64 (5), B64 (6), B64 (7),
-  B64 (8), B64 (9), B64 (10), B64 (11),
-  B64 (12), B64 (13), B64 (14), B64 (15),
-  B64 (16), B64 (17), B64 (18), B64 (19),
-  B64 (20), B64 (21), B64 (22), B64 (23),
-  B64 (24), B64 (25), B64 (26), B64 (27),
-  B64 (28), B64 (29), B64 (30), B64 (31),
-  B64 (32), B64 (33), B64 (34), B64 (35),
-  B64 (36), B64 (37), B64 (38), B64 (39),
-  B64 (40), B64 (41), B64 (42), B64 (43),
-  B64 (44), B64 (45), B64 (46), B64 (47),
-  B64 (48), B64 (49), B64 (50), B64 (51),
-  B64 (52), B64 (53), B64 (54), B64 (55),
-  B64 (56), B64 (57), B64 (58), B64 (59),
-  B64 (60), B64 (61), B64 (62), B64 (63),
-  B64 (64), B64 (65), B64 (66), B64 (67),
-  B64 (68), B64 (69), B64 (70), B64 (71),
-  B64 (72), B64 (73), B64 (74), B64 (75),
-  B64 (76), B64 (77), B64 (78), B64 (79),
-  B64 (80), B64 (81), B64 (82), B64 (83),
-  B64 (84), B64 (85), B64 (86), B64 (87),
-  B64 (88), B64 (89), B64 (90), B64 (91),
-  B64 (92), B64 (93), B64 (94), B64 (95),
-  B64 (96), B64 (97), B64 (98), B64 (99),
-  B64 (100), B64 (101), B64 (102), B64 (103),
-  B64 (104), B64 (105), B64 (106), B64 (107),
-  B64 (108), B64 (109), B64 (110), B64 (111),
-  B64 (112), B64 (113), B64 (114), B64 (115),
-  B64 (116), B64 (117), B64 (118), B64 (119),
-  B64 (120), B64 (121), B64 (122), B64 (123),
-  B64 (124), B64 (125), B64 (126), B64 (127),
-  B64 (128), B64 (129), B64 (130), B64 (131),
-  B64 (132), B64 (133), B64 (134), B64 (135),
-  B64 (136), B64 (137), B64 (138), B64 (139),
-  B64 (140), B64 (141), B64 (142), B64 (143),
-  B64 (144), B64 (145), B64 (146), B64 (147),
-  B64 (148), B64 (149), B64 (150), B64 (151),
-  B64 (152), B64 (153), B64 (154), B64 (155),
-  B64 (156), B64 (157), B64 (158), B64 (159),
-  B64 (160), B64 (161), B64 (162), B64 (163),
-  B64 (164), B64 (165), B64 (166), B64 (167),
-  B64 (168), B64 (169), B64 (170), B64 (171),
-  B64 (172), B64 (173), B64 (174), B64 (175),
-  B64 (176), B64 (177), B64 (178), B64 (179),
-  B64 (180), B64 (181), B64 (182), B64 (183),
-  B64 (184), B64 (185), B64 (186), B64 (187),
-  B64 (188), B64 (189), B64 (190), B64 (191),
-  B64 (192), B64 (193), B64 (194), B64 (195),
-  B64 (196), B64 (197), B64 (198), B64 (199),
-  B64 (200), B64 (201), B64 (202), B64 (203),
-  B64 (204), B64 (205), B64 (206), B64 (207),
-  B64 (208), B64 (209), B64 (210), B64 (211),
-  B64 (212), B64 (213), B64 (214), B64 (215),
-  B64 (216), B64 (217), B64 (218), B64 (219),
-  B64 (220), B64 (221), B64 (222), B64 (223),
-  B64 (224), B64 (225), B64 (226), B64 (227),
-  B64 (228), B64 (229), B64 (230), B64 (231),
-  B64 (232), B64 (233), B64 (234), B64 (235),
-  B64 (236), B64 (237), B64 (238), B64 (239),
-  B64 (240), B64 (241), B64 (242), B64 (243),
-  B64 (244), B64 (245), B64 (246), B64 (247),
-  B64 (248), B64 (249), B64 (250), B64 (251),
-  B64 (252), B64 (253), B64 (254), B64 (255)
-};
-
-#define isbase64(x) ((to_uchar (x) <= 255) && (0 <= b64[to_uchar (x)]))
-
-
-/**
- * utility_build_decoded_id
- *
- * Reverse of utility_build_encoded_id()
- */
-gchar *
-utility_build_decoded_id (const gchar *prefix, const gchar *id)
-{
-	gchar *str;
-	gchar *out;
-	const gchar *in;
-	int ln = 0;
-	int offset;
-
-	/* go to the first byte of actual data */
-	in = id;
-	if (prefix) {
-		const gchar *tmp;
-		tmp = prefix;
-		while (*tmp) {
-			in++;
-			tmp++;
-		}
+		gtk_window_set_resizable (GTK_WINDOW (dlg), TRUE);
 	}
 
-	ln = (strlen (in) * 3) / 4 + 3;
-	str = g_new0 (char, ln);
-	out = str;
+	return dlg;
+}
 
-	/* create data */
-	offset = 3;
-	for (; offset == 3; in += 4) {
-		offset = 0;
-		
-		if (isbase64 (in[0]) && isbase64 (in[1])) {
-			out[0] = ((b64 [to_uchar (in[0])] << 2) | 
-				  (b64 [to_uchar (in[1])] >> 4));
-			
-			if (isbase64 (in[2])) {
-				out[1] = (((b64 [to_uchar (in[1])] << 4) & 0xf0) |
-					  (b64 [to_uchar (in[2])] >> 2));
+/**
+ * gnome_db_utility_display_error_with_keep_or_discard_choice
+ * @form:
+ * @filled_error: a #GError containing the error to display
+ *
+ * Displays a dialog showing @filled_error's message and asks the user to either keep the data which
+ * led to the error, or to discard it.
+ *
+ * Returns: TRUE if current data can be discarded
+ */
+gboolean
+gnome_db_utility_display_error_with_keep_or_discard_choice (GnomeDbDataWidget *form, GError *filled_error)
+{
+	GtkWidget *dlg;
+	gint res;
 
-				if (isbase64 (in[3])) {
-					out[2] = (((b64 [to_uchar (in[2])] << 6) & 0xc0) |
-						  b64 [to_uchar (in[3])]);
-					offset = 3;
-				}
-			}
-		}
+	if (filled_error && (filled_error->domain == GDA_DATA_PROXY_ERROR) &&
+	    (filled_error->code == GDA_DATA_PROXY_COMMIT_CANCELLED))
+		return FALSE;
 
-		out += offset;
-        }
+	dlg = create_data_error_dialog (form, TRUE, TRUE, filled_error);
+	gtk_dialog_add_buttons (GTK_DIALOG (dlg),
+				_("Discard modified data"), GTK_RESPONSE_REJECT,
+				_("Correct data first"), GTK_RESPONSE_NONE, NULL);
+	res = gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
+	return (res == GTK_RESPONSE_REJECT) ? TRUE : FALSE;
+}
 
-	return str;
+/**
+ * gnome_db_utility_display_error
+ * @form:
+ * @filled_error: a #GError containing the error to display
+ *
+ * Displays a dialog showing @filled_error's message and asks the user to either keep the data which
+ * led to the error.
+ */
+void
+gnome_db_utility_display_error (GnomeDbDataWidget *form, gboolean can_discard, GError *filled_error)
+{
+	GtkWidget *dlg;
+	
+	if (filled_error && (filled_error->domain == GDA_DATA_PROXY_ERROR) &&
+	    (filled_error->code == GDA_DATA_PROXY_COMMIT_CANCELLED))
+		return;
+
+	dlg = create_data_error_dialog (form, FALSE, can_discard, filled_error);
+	gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
 }
